/****************************************************************************
   Copyright (C) 2008-2009, Orcan Ogetbil. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

// Implementation of board functions
#include <iostream>
#include <cstdlib>
#include <cstddef>
#include <ctime>
#include <vector>


#include "board.h"

namespace blueangel
{
  Board::Board(int size) 
  {
    node = NULL;
    top = new BoardFrame;
    dimension = size;
    srand((unsigned)time(0));

    cout << "creating the board" << endl;
    top->el.resize(size, vector< int >(size,0));

    top->childBoard.resize(size);

    childScore.resize(size);
    
    for (int index1 =0; index1 < dimension; index1++)
      {
	for (int index2 =0; index2 < dimension; index2++)
	  {
	    int random_integer =  rand()%18;
	    if (random_integer > 8)
	      {
		top->el[index1][index2] = 8 - random_integer;
	      }
	    else
	      {
		top->el[index1][index2] = random_integer + 1;
	      }
	    
	  }
      }    
  }

  Board::~Board()
  {
    //    cout << "deleting top";
    //    delete top;
  }

  void Board::deleteTop()
  {
    deleteTop(top);
  }

  void Board::deleteTop(BoardFrame *&leaf)
  {
    delete leaf;
  }

  int Board::getBestChild()
  {
    return getBestChild(node);
  }

  int Board::getBestChild(ScoreBoard *leaf)
  {
    if (leaf == NULL) cout << "LEAF is NULL!!!"<<flush;
    int best = leaf->childNumber;
    if (leaf->right != NULL)
      best = getBestChild(leaf->right);
    return best;
  }

  void Board::scoreSort() //public
  // dimension is a private variable, determined during runtime.
  // node, a private variable, is a pointer pointing to the root of a binary tree,
  // it is NULL at this point.
  // childScore is a std::vector of dimension "dimension", it is private.
  {
    for (int index=0; index < dimension; index++)
      {
	cout << "sorting child " << index << endl;
	//node = NULL;
	//delete node;
	scoreSort(node, childScore[index],index);
	//	delete node;
	//	if ( node == NULL) cout << "Node is NULL again ?? WTF??"<< endl << flush;
      }
  }

  void Board::scoreSort(ScoreBoard *&leaf, int val, int child) //private
  {
    if ( leaf == NULL)
      {
	leaf = new ScoreBoard(val,child);
	cout << "child is " << leaf->childNumber << "with value " << leaf->value << "  "<< endl << flush;
      }
    else if (val < leaf->value)
      {
	//cout << "val " << val << " less than leaf->value " << leaf->value << endl;
	scoreSort(leaf->left, val, child);
      }
    else
      {
	//cout << "val " << val << " more than or equal to leaf->value " << leaf->value << endl;
	scoreSort(leaf->right, val, child);
      }
    // TODO: write the destructor
  }

  void  Board::clearScores()
  {
    clearScores(node);
  }

  void  Board::clearScores(ScoreBoard *&leaf)
  {
    if (leaf != NULL)
      {
	clearScores(leaf->left);
	clearScores(leaf->right);
	delete leaf;
	//cout << "delete score successful!" << endl;
      }
  }

  void Board::scoreInsert(int value, int score1, int score2, int childNumber, int uniqueness, int ends=-1)
  // This function is the heart of the AI. It interprets all the analysis
  {
    childScore[childNumber] = 0;
    //cout << "for childNumber: " << childNumber << ", score1: " << score1 << " score2: " << score2 << " value: " << value << " ends: " << ends << endl;
    if ( ends == 0 ) // if the game ends on a single next move
      {
	if (score2 + value > score1)
	  {
	    childScore[childNumber] = 500;
	    cout << "VOILAA!!!" << endl;
	  }
	else
	  childScore[childNumber] = -500;
      }
    
    else if ( ends ==1 ) // if the game will end in two moves, with the current setup this is useless
      {
	if (score2 + value > score1)
          childScore[childNumber] = value+20;
        else
          childScore[childNumber] = value-20;
      }

    else if ( ends ==2 ) // if the game will end in three moves
      {
	cout << endl<<"there is a unique move" << uniqueness << endl << flush;
        if (score2 + value > score1)
	  {
	    if (uniqueness > 1)
	      childScore[childNumber] = 500;
	    else
	      childScore[childNumber] = value+5;
	  }
        else
	  {
	    if (uniqueness > 1)
	      childScore[childNumber] = -500;
	    else
	      childScore[childNumber] = value-5;
	  }
      }

    else if ( ends == -2 ) // if the element is 0, I can't pick that by the games rules
      childScore[childNumber] = -10000;

    //else if (ends == -1 and childScore[childNumber] == 0) // if it is a regular move
    //  childScore[childNumber] = value;
    
    else if (ends == -1) // if it is a regular move
      childScore[childNumber] = value;

  }
    
  void Board::bestValue(int score1, int score2, int coord_row, int coord_col, int direction)
  {
    bestValue(top,score1,score2, coord_row, coord_col, direction,0,-1,0); //FIXME: last three values ???
  }

  int Board::bestValue(BoardFrame *leaf, int score1, int score2, int coord_row, int coord_col, int direction, int uniqueness=0, int topchild=-1, int valuesofar=0)
  {
    int bestvalue = -999;
    leaf->childBoard.resize(dimension);
    if (direction%2==0 and direction < depth)
      {
	//	feedChildren(leaf,coord_row,coord_col,direction);
	if (isUnique(leaf,2,coord_row)) 
	  uniqueness++;
	else
	  uniqueness=0;
	
	for (int childNumber=0; childNumber < dimension; childNumber++)
	  {	    
	    if (direction == 0) 
	      {
		topchild = childNumber;
		bestvalue = -999;
	      }
	    leaf->childBoard[childNumber]= new BoardFrame;
	    leaf->childBoard[childNumber]->el.resize(dimension, vector< int >(dimension,0));
	    for (int col=0; col < dimension; col++)
              {
                for (int row=0; row < dimension; row++)
                  {
                    int val = leaf->el[row][col];
                    leaf->childBoard[childNumber]->el[row][col]= val;
                    if (col == childNumber and row == coord_row)
                      {
                        leaf->childBoard[childNumber]->el[row][col]= 0;
                      }
                  }
              }
            if (leaf->el[coord_row][childNumber] == 0)
              {
		if (direction == 0)
		  scoreInsert(0,score1, score2,topchild,uniqueness,-2);
		delete  leaf->childBoard[childNumber];
		continue;
	      }
	    
	    bool isfree = isFree(leaf->childBoard[childNumber],1,childNumber);

	    //	    cout << "level: "<< direction <<". for child " << childNumber << " with value " << leaf->el[coord_row][childNumber]<< ", uniqueness is " << uniqueness << endl << flush;
   
   	    int sofar = leaf->el[coord_row][childNumber] - valuesofar;
	    
	    if (isfree == false and !(leaf->el[coord_row][childNumber]==0) )
	      {
		scoreInsert(sofar, score1, score2, topchild, uniqueness,direction);
		continue;
	      }

	    int childbest = bestValue(leaf->childBoard[childNumber], score1, score2, coord_row, childNumber, direction+1, uniqueness, topchild, sofar);
	    if (direction+1 == depth)
	      {
		if ( (leaf->el[coord_row][childNumber] > bestvalue) and !(leaf->el[coord_row][childNumber]==0) )
		  {
		    bestvalue = (leaf->el[coord_row][childNumber]);
		    
		    if (direction == 0)
		      {
			scoreInsert(bestvalue, score1, score2, topchild,uniqueness);
			continue;
			//			coord_col = childNumber;
		      }
		  }
	      }
	    else if ( leaf->el[coord_row][childNumber] - childbest > bestvalue and !(leaf->el[coord_row][childNumber]==0))
	      {
		bestvalue = leaf->el[coord_row][childNumber] - childbest;

		if (direction == 0)
		  {
		    scoreInsert(bestvalue, score1, score2,topchild, uniqueness);
		    continue;
		    //		    coord_col = childNumber;
		  }
	      }
	    delete  leaf->childBoard[childNumber];
	    //starveChildren(leaf->childBoard[childNumber]);
	    //if (direction == depth-1)
	    //	scoreInsert(sofar, score1, score2, topchild);
	    if (direction == 0)
	      cout << (100 * childNumber/ dimension) << "% done." << endl;

	  }
      }
    else if (direction%2==1 and direction < depth)
      {
	//	feedChildren(leaf,coord_row,coord_col,direction);
	if (isUnique(leaf,1,coord_col))
	  uniqueness++;
	else
	  uniqueness=0;

	for (int childNumber=0; childNumber < dimension; childNumber++)
	  {
	    if (direction == 0)
              {
                bestvalue = -999;
              }
	    //	    int bestvalue = -999;
	    leaf->childBoard[childNumber]= new BoardFrame;
	    leaf->childBoard[childNumber]->el.resize(dimension, vector< int >(dimension,0));
	    for (int row=0; row < dimension; row++)
              {
                for (int col=0; col < dimension; col++)
                  {
                    int val = leaf->el[row][col];
                    leaf->childBoard[childNumber]->el[row][col]= val;
                    if(col == coord_col and row == childNumber )
                      {
                        leaf->childBoard[childNumber]->el[row][col]=0;
                      }
                  }
              }
            if (leaf->el[childNumber][coord_col] == 0)
              {
		delete  leaf->childBoard[childNumber];
                continue;
              }

	    bool isfree = isFree(leaf->childBoard[childNumber],2,childNumber);


	    int sofar=leaf->el[childNumber][coord_col] - valuesofar;

            if (isfree == false and !(leaf->el[childNumber][coord_col]==0) )
	      {
		scoreInsert(sofar, score1, score2, topchild, uniqueness,direction);
		continue;
	      }

	    int childbest = bestValue(leaf->childBoard[childNumber], score1, score2, childNumber, coord_col, direction+1, uniqueness, topchild, sofar);
	    if (direction+1 == depth)
	      {
		if ( (leaf->el[childNumber][coord_col] > bestvalue) and !(leaf->el[childNumber][coord_col]==0) )
		  {
		    bestvalue = (leaf->el[childNumber][coord_col]);
		    //  cout << "bestvalue is " << bestvalue << endl<< flush;
		    //		    scoreInsert(sofar, score1, score2, topchild);
		  }
	      }
	    else if ( leaf->el[childNumber][coord_col] - childbest > bestvalue and !(leaf->el[childNumber][coord_col]==0))
	      {
		bestvalue = leaf->el[childNumber][coord_col] - childbest;
		//		cout << "bestvalue is " << bestvalue << endl<< flush;
		// scoreInsert(sofar, score1, score2, topchild);
	      }
	    //if (leaf->childBoard[childNumber]!=NULL) 
	    //starveChildren(leaf->childBoard[childNumber]);
	    delete  leaf->childBoard[childNumber];
	
	    //	    if (direction == 0)
	    //  scoreInsert(sofar, score1, score2, topchild);

	  }
      }
    //    if ( bestvalue == -999) cout << "THIS SHOULDNT HAPPEN!";
    //    if (direction == 1) cout << "value returned " << bestvalue << endl<< flush;
    if ( bestvalue == -999) bestvalue = 0;
    return bestvalue;
  }

  bool Board::isFree(BoardFrame *leaf, int player, int coord)
  {
    bool free = true;
    int tot = 0;
    if (player == 1)
      {
	for (int index = 0; index < dimension; index++)
	  {
	    tot = tot + abs( leaf->el[index][coord] );
	  }
      }
    else
      {
	for (int index = 0; index < dimension; index++)
	  {
	    tot = tot + abs ( leaf->el[coord][index] );
	  }
      }
    if (tot == 0)
      free = false;
    return free;
  }

  bool Board::isUnique(BoardFrame *leaf, int player, int coord)
  {
    bool unique = false;
    int no_of_possible_moves = 0;
    if (player == 1)
      {
        for (int index = 0; index < dimension; index++)
          {
	    if (leaf->el[index][coord] != 0)
	      no_of_possible_moves++;
	  }
      }
    else
      {
	for (int index = 0; index < dimension; index++)
	  {
	    if (leaf->el[coord][index] != 0)
	      {
		no_of_possible_moves++;
	      }
          }
      }
    if ( leaf == top ) cout << "number of possible moves for row "<< coord << " is " << no_of_possible_moves<< endl;
    if ( no_of_possible_moves == 1 ) unique = true;
    return unique;
  }

  void Board::setDepth(int value)
  {
    depth = value;
  }
	       
  int Board::getDimension() const
  {
    return dimension;
  }

  void Board::setDimension(int dim)
  {
    dimension = dim;
  }

  void Board::setElement(int row, int col, int value)
  {
    top->el[row][col]=value;
  }

  int Board::getElement(int row, int col) const
  {
    return top->el[row][col];
  }

  void Board::printBoard(int player, int coord)
  {
    //ANSI escape codes
    char blue[] = { 0x1b, '[', '1', ';', '3', '4', 'm', 0 };
    char white[] = {0x1b, '[', '1', ';', '3', '7', 'm', 0};
    char normal[] = { 0x1b, '[', '0', ';', '3', '9', 'm', 0 };
    char red[] = {0x1b, '[', '1', ';', '3', '1', 'm', 0};
    char darkred[] = {0x1b, '[', '0', ';', '3', '1', 'm', 0};
    char green[] = {0x1b, '[', '1', ';', '3', '2', 'm', 0};

    char a[10]={'a','b','c','d','e','f','g','h','i','j'};
    char b[10]={'0','1','2','3','4','5','6','7','8','9'};

    cout << green << "  -" << normal;
    for (int index1 =0; index1 < dimension; index1++)
    {
      cout << green <<"--" << normal;
    }
    cout << green << "--" << normal << endl;

    for (int index1 =0; index1 < dimension; index1++)
      {
	if (player == 2 and index1 == coord)
	  cout << white << a[index1] << green<< " | " <<normal;
	else
	  cout << a[index1] << green<< " | " <<normal;
        for (int index2 =0; index2 < dimension; index2++)
          {
	    if (top->el[index1][index2] == 0)
	      cout << blue << 0 << normal << " " ;
	    else if (player == 2 and index1 == coord)
	      {
		if (top->el[index1][index2] < 0)
		  cout << red << - top->el[index1][index2] << normal << " " ;
		else
		  cout << white << top->el[index1][index2] << normal << " " ;
	      }
	    else if (player == 1 and index2 == coord)
	      {
		if (top->el[index1][index2] < 0)
                  cout << red << - top->el[index1][index2] << normal << " " ;
                else
		  cout << white << top->el[index1][index2] << normal << " " ;
	      }
	    else if (top->el[index1][index2] < 0)
	      cout << darkred << - top->el[index1][index2] << normal << " " ;
	    else
	      cout << top->el[index1][index2] << " " ;
          }
        cout << green << "|" << normal <<endl;
      }
    cout << green << "  -" << normal;
    for (int index1 =0; index1 < dimension; index1++)
    {
      cout << green <<"--"<<normal;
    }
    cout << green<< "--"<<normal <<endl<<"    ";
    for (int index2 =0; index2 < dimension; index2++)
      {
	if (player == 1 and index2 == coord)
	  cout << white << b[index2] << normal <<" ";
	else
	  cout << b[index2] <<" ";
      }
    cout << endl;
  }
}
