/****************************************************************************
   Copyright (C) 2008-2009, Orcan Ogetbil. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

#ifndef BOARD_H
#define BOARD_H

// Basic command-line blue angel game
#include <iostream>
#include <vector>
using namespace std;

namespace blueangel
{
  struct ScoreBoard
  {
    // A binary tree that is sorted by the "value" of each childNode.
    // AI picks the node with the highest value.

    int value;       // The value of a branch.
    int childNumber; // The number of the branch.
    //    int ends;        // game ends on this branch in the ends'th iteration. 
                     // 0 means the game doesn't end in this branch.
    //    int endsvalue;   // if ends>0 this is the value of the ending move.
    ScoreBoard* left;
    ScoreBoard* right;
    //ScoreBoard(int b_value=0, int b_childNumber=0, int b_ends=0) 
    ScoreBoard(int b_value=0, int b_childNumber=0) 
    {
      // Constructor.  Make a node containing value, childNumber.
      value = b_value;
      childNumber = b_childNumber;
      left = NULL;
      right = NULL;
    }

  };

  struct BoardFrame
  {
    vector< vector< int > > el;
    // Holds the values of the elements of the board

    vector< BoardFrame* > childBoard;    
    // Child nodes
  };

  class Board
  {
  public:
    Board(int size);
    // Initializes the board by setting the size as size x size

    //Board(const Board& parentBoard);
    // Copy constructor

    ~Board();
    // Destroy me babe.

    void printBoard(int player, int coord);
    // Outputs the board

    void setDimension(int dim);
    //sets the dimension of the board

    int getDimension() const;
    // outputs the dimension of the board

    int getElement(int row, int col) const;
    // Gets an element from the board

    void setElement(int row, int col, int value);
    //Sets an element to the specified value

    //void starveChildren();
    // delete the child nodes to free up memory

    //    void feedChildren(int coord_row, int coord_col, int direction);
    // fills the children boards.
    // if direction is even, start feeding children in the current coord_row
    // if direction is odd, start feeding children in the current coord_col

    void bestValue(int score1, int score2, int coord_row, int coord_col, int direction);
    // returns the best value to pick for AI
    // more importantly, sets coord_col for the best value

    void setDepth(int value);
    // sets number of iterations for AI

    void scoreSort();

    int getBestChild();

    void clearScores();

    void deleteTop();
    ScoreBoard* node;

    //Set these two back to private:
    BoardFrame* top;
    //this is the pointer to the actual board
    bool isFree(BoardFrame *leaf, int player, int coord);
    // does the leaf have a valid move?


  private:
    vector< int > childScore;
    // holds the best value of each child. The information will be
    // sorted in the ScoreBoard struct from lowest to highest.



    void deleteTop(BoardFrame *&leaf);

    void clearScores(ScoreBoard *&leaf);

    void scoreSort(ScoreBoard *&leaf, int val, int child);

    int getBestChild(ScoreBoard *leaf);
    // returns the column with highest value


    void scoreInsert(int value, int score1, int score2, int childNumber, int uniqueness, int ends);

    int depth;
    // Number of iterations for AI

    int dimension;
    //declare dimension

    bool isUnique(BoardFrame *leaf, int player, int coord);
    // is there just only one possible move for the leaf?

    int bestValue(BoardFrame *leaf, int score1, int score2, int coord_row, int coord_col, int direction, int uniqueness, int topchild, int valuesofar);
    // returns the best value to pick for AI
    // more importantly, sets coord_col for the best value


  };//class Board

}// namespace blueangel

#endif
