/****************************************************************************
   Copyright (C) 2008-2009, Orcan Ogetbil. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

#include <ctime>
#include <KApplication>
#include <KAction>
#include <KLocale>
#include <KActionCollection>
#include <KStandardAction>
#include <KConfigDialog>
#include <iostream>
#include <cstdlib>
#include <cstdio>

#include "mainwindow.h"
#include "general.h"
#include "settings.h"

using namespace std;
using namespace blueangel;


void MainWindow::updateSettings()
{
  int olddimension = m_melek->mdimension;
  m_melek->players = Settings::players();
  m_melek->mdimension = Settings::size();
  //m_melek->difficulty = Settings::difficulty();
  //m_board->setDepth(difficulty);
  m_melek->setDifficulty(Settings::difficulty());
  

  Settings::self()->writeConfig();
  
  if (olddimension != m_melek->mdimension)
    {
      m_melek->restartGame(olddimension);
      //restartGame();
    }
}

void MainWindow::showSettings()
{
  if (KConfigDialog::showDialog("settings"))
    return;
  
  KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
  General *mGeneral = new General();// 0, "Normal Settings" );

  dialog->addPage( mGeneral, i18n("General"), "settings" );
  connect( dialog, SIGNAL(settingsChanged(const QString &)), this, SLOT(updateSettings()) );
  dialog->show();
}


MainWindow::MainWindow(QWidget *parent) : KXmlGuiWindow(parent)
{
  int s_players = Settings::players();
  int s_dimension = Settings::size();
  int s_difficulty = Settings::difficulty();

  QGroupBox *mainGroup = new QGroupBox();
  QGroupBox *scoresGroup = new QGroupBox(mainGroup);

  butgrp = new KButtonGroup(mainGroup);
  length = 60;
  spacing = 20;
  int containersize = s_dimension * (length + spacing) + spacing;
  containersize=660;
  
  butgrp->setFixedSize (containersize, containersize );

  player1scorebox = new QGroupBox(scoresGroup);//this);
  //player1scorebox->move(containersize,25);
  player1scorebox->setTitle("Player 1:");
  player1scorebox->setFixedSize(140,140);

  QPalette palette1;
  palette1.setColor(QPalette::Normal, QPalette::Foreground, Qt::yellow);
  palette1.setColor(QPalette::Normal, QPalette::Background, Qt::black);
  palette1.setColor(QPalette::Normal, QPalette::Light, Qt::yellow);
  palette1.setColor(QPalette::Normal, QPalette::Dark, Qt::darkYellow);

  QPalette palette2;
  palette2.setColor(QPalette::Normal, QPalette::Foreground, Qt::blue);
  palette2.setColor(QPalette::Normal, QPalette::Background, Qt::black);
  palette2.setColor(QPalette::Normal, QPalette::Light, Qt::blue);
  palette2.setColor(QPalette::Normal, QPalette::Dark, Qt::darkBlue);

  lcdNumber1 = new QLCDNumber(player1scorebox);
  lcdNumber1->setSegmentStyle( QLCDNumber::Flat );
  lcdNumber1->setAutoFillBackground(true);
  lcdNumber1->setPalette(palette1);
  lcdNumber1->setGeometry(QRect(10,30,120,80));

  player2scorebox = new QGroupBox(scoresGroup);//this);
  //player2scorebox->move(containersize,165);
  player2scorebox->setTitle("Player 2:");
  player2scorebox->setFixedSize(140,140);

  lcdNumber2 = new QLCDNumber(player2scorebox);
  lcdNumber2->setSegmentStyle( QLCDNumber::Flat );
  lcdNumber2->setAutoFillBackground(true);
  lcdNumber2->setPalette(palette2);
  lcdNumber2->setGeometry(QRect(10,30,120,80));


  QVBoxLayout *scoresLayout = new QVBoxLayout;
  scoresLayout->addWidget(player1scorebox);
  scoresLayout->addWidget(player2scorebox);

  scoresGroup->setFixedWidth(150);
  scoresGroup->setLayout(scoresLayout);

  QHBoxLayout *mainLayout = new QHBoxLayout;
  butgrp->setFlat(true);
  scoresGroup->setFlat(true);
  mainLayout->addWidget(butgrp);
  mainLayout->addWidget(scoresGroup);
  //mainLayout->addWidget(player1scorebox);
  //mainLayout->addWidget(player2scorebox);

  mainGroup->setLayout(mainLayout);  

  m_melek = new MelekGame(butgrp, lcdNumber1, lcdNumber2, length, s_dimension);

  m_melek->players = s_players;
  m_melek->mdimension = s_dimension;
  m_melek->difficulty = s_difficulty;
  
  setCentralWidget(mainGroup);

  setupActions();
  //  newaction = false;
}


void MainWindow::restart()
{
  //delete m_melek;
  //m_melek = new MelekGame(butgrp, lcdNumber1, lcdNumber2, 20, 8);
  m_melek->restartGame();
}



void MainWindow::setupActions()
{
  KAction* newAction = new KAction(this);
  newAction->setText(i18n("New"));
  newAction->setIcon(KIcon("dialog-ok"));
  newAction->setShortcut(Qt::CTRL + Qt::Key_N);
  actionCollection()->addAction("new", newAction);
  connect(newAction, SIGNAL(triggered(bool)),
          this, SLOT(restart()));
 
  KAction* prefAction = new KAction(this);
  prefAction->setText(i18n("Settings"));
  prefAction->setIcon(KIcon("configure"));
  prefAction->setShortcut(Qt::CTRL + Qt::Key_S);
  actionCollection()->addAction("settings", prefAction);
  connect(prefAction, SIGNAL(triggered(bool)),
          this, SLOT(showSettings()));

  KStandardAction::quit(kapp, SLOT(quit()),
                        actionCollection());
 
  KStandardAction::preferences(this, SLOT(showSettings()),
  		  actionCollection());

  setupGUI();
  }
