/****************************************************************************
   Copyright (C) 2008-2009, Orcan Ogetbil. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

//#include <QSignalMapper>
//#include <QGridLayout>
//#include <kbuttongroup.h>
#include <kpushbutton.h>
//#include <QLabel>
#include <QFont>
#include "settings.h"
#include "melekgame.h"
//#include "board.h"
#include <cstdio>

using namespace std;
using namespace blueangel;

void MelekGame::setScore(int player1score, int player2score)
{
  score1 = player1score;
  score2 = player2score;
  mlcd1->display(score1);
  mlcd2->display(score2);
}

void MelekGame::setDifficulty(int depth)
{
  difficulty = depth;
  m_board->setDepth(difficulty);
}

void MelekGame::restartGame(int oldsize)
{
  for (int index1 =0; index1 < oldsize; index1++)
    {
      for (int index2 =0; index2 < oldsize; index2++)
        {
          signalMapper->removeMappings(buttonell[index1][index2]);
	  delete buttonell[index1][index2];
	  buttonell[index1][index2] = NULL;
          //buttonell[index1][index2]->hide();
        }
    }

  delete buttonsLayout;
  gameOverString->hide();
  newGame();
}

void MelekGame::restartGame()
{
  for (int index1 =0; index1 < mdimension; index1++)
    {
      for (int index2 =0; index2 < mdimension; index2++)
        {
          signalMapper->removeMappings(buttonell[index1][index2]);
	  delete buttonell[index1][index2];
	  buttonell[index1][index2] = NULL;
        }
    }
    
  //delete signalMapper;
  //delete m_board;
  delete buttonsLayout;
  gameOverString->hide();
  newGame();
}

MelekGame::MelekGame(KButtonGroup *buttonGroup, QLCDNumber *lcd1, QLCDNumber *lcd2, int length, int dim)
{
  temprow = 99;
  tempcol = 99;
  score1=0;
  score2=0;

  mbutgrp = buttonGroup;
  mlcd1 = lcd1;
  mlcd2 = lcd2;
  mlength = length;
  mdimension=dim;

  gameOverString = new QLabel(mbutgrp);

  QFont gofont("Helvetica", 52, QFont::Bold);
  //QLabel *mystring = new QLabel(QString("YARRRRRRRRRAAA"),mbutgrp);
  gameOverString->setGeometry(50,300,570,60);
  gameOverString->setFont(gofont);

  newGame();
}

void MelekGame::newGame()
{
  turn=1;

  //cout << "RESTARTING with dimension" << mdimension << endl << flush;
  //m_melek->players = Settings::players();
  //m_melek->dimension = Settings::size();
  //m_melek->difficulty = Settings::difficulty();
  //m_board->setDepth(difficulty);
  difficulty = Settings::difficulty();



  srand((unsigned)time(0));
  //buttonell.resize(mdimension, vector< KPushButton* >(mdimension));
  buttonell.resize(10, vector< KPushButton* >(10));

  m_board = new Board(mdimension);

  m_board->setDepth(difficulty);
  setScore(0,0);

  signalMapper = new QSignalMapper(this);
  //signalMapper = new QSignalMapper();

  QFont butfont("Helvetica", 200/mdimension, QFont::Normal);

  buttonsLayout = new QGridLayout;

  for (int index1 =0; index1 < mdimension; index1++)
    {
      for (int index2 =0; index2 < mdimension; index2++)
        {
          int random_integer = m_board->getElement(index1,index2);
          
	  buttonell[index1][index2] = new KPushButton(mbutgrp);


          //if (buttonell[index1][index2]->minimumSize().isNull())
          //  {
	  //buttonell[index1][index2]->setMinimumSize ( length, length );
	      //buttonell[index1][index2]->setFixedSize ( mlength, mlength );
	  buttonell[index1][index2]->setFixedSize ( 640/mdimension, 640/mdimension);
	  //  }
          //buttonell[index1][index2]->move(20 + (80) * index1, 20 + (80) * index2);
          buttonsLayout->addWidget(buttonell[index1][index2], index1, index2);

          buttonell[index1][index2]->setText(QString::number(random_integer));
	  buttonell[index1][index2]->setFont(butfont);
          if (random_integer < 0)
            {
              buttonell[index1][index2]->setPalette(QPalette(Qt::red));
            }
          else
            {
              buttonell[index1][index2]->setPalette(QPalette(Qt::green));
            }
	  
          QString tbs;
          tbs = QString::number(index1)+","+QString::number(index2)+","+buttonell[index1][index2]->text();
          connect(buttonell[index1][index2], SIGNAL(clicked()), signalMapper, SLOT(map()));
	  
	  signalMapper->setMapping(buttonell[index1][index2], tbs);
        }
    }
  
  mbutgrp->setLayout(buttonsLayout);

  connect(signalMapper, SIGNAL(mapped(QString)), this, SLOT(clicked(QString)));
}

void MelekGame::clicked(const QString &text)
{
  QStringList list1 = text.split(",");
  
  cout << "sent: " << endl;
  printf("%s \n", qPrintable(text));
  fflush(stdout);
  //newaction = true;
  gameOn(list1.at(0).toInt(),list1.at(1).toInt(),list1.at(2).toInt());
}

void MelekGame::gameOn(int index1, int index2, int value)
{
  int isfree;
  if (m_board->getElement(index1,index2) == 0)
    {
      cout << "Can't pick 0 element!" << endl;
      return;
    }

  if (turn%2) // if Player 1 is on
    {

      if (index2 != tempcol and turn > 1)
        {
          cout << "Wrong Column! You should use column " << tempcol << endl;
          return;
        }

      temprow = index1;

      buttonell[index1][index2]->setText(QString::number(0));
      setScore(score1+value,score2);
      colorize(turn+1,index1);
      isfree = m_board->isFree(m_board->top,2,temprow);
    }
  else // if Player 2 is on
    {
      if (index1 != temprow)
        {
          cout << "Wrong Row! You should use row " << temprow << endl;
          return;
        }

      buttonell[index1][index2]->setText(QString::number(0));
      tempcol = index2;

      setScore(score1,score2+value);
      colorize(turn+1,index2);
      isfree = m_board->isFree(m_board->top,1,tempcol);
    }


  m_board->setElement(index1,index2,0);
  turn++;
  if (players == 1)
    {
      int element = getElementFromAI(index1, index2);
      m_board->setElement(index1,index2,0);
      buttonell[index1][index2]->setText(QString::number(0));
      setScore(score1,score2+element);
      colorize(turn+1,index2);
      tempcol = index2;
      turn++;
      isfree = m_board->isFree(m_board->top,1,tempcol);
    }

  if (isfree == 0)
    {
      //cout << "GAME OVER!" << endl << flush;
      gameOver();
    }
}//gameOn()

void MelekGame::gameOver()
{
  if (score1 > score2)
    gameOverString->setText("PLAYER 1 WINS!");
  else if (score2 > score1)
    gameOverString->setText("PLAYER 2 WINS!");
  else
    gameOverString->setText("IT'S A TIE!");

  gameOverString->show();
  gameOverString->raise();
}

int MelekGame::getElementFromAI(int& coord_row, int& coord_col)
{

  int direction = 0; // when even, children are filled in the rows
                     // when  odd, children are filled in the columns
  // First fill in the child boards:

  //  m_board.feedChildren(coord_row, coord_col, direction);
  int bestcolvalue;
  //cout << "feeding AI with column: " << coord_col << " and row: " << coord_row << endl;
  // FIXME:
  m_board->node = NULL;
  m_board->bestValue(score1,score2,coord_row,coord_col,direction);
  m_board->scoreSort();
  int bestcol =  m_board->getBestChild();
  coord_col = bestcol;
  bestcolvalue= m_board->getElement(coord_row,coord_col);
  cout << "best column is " << bestcol<< endl << flush;
  m_board->clearScores();

  return bestcolvalue;
}

void MelekGame::colorize(int playerturn, int element)
{
  cout << "TURN: " << playerturn << endl;
  for (int index1 =0; index1 < mdimension; index1++)
    {
      for (int index2 =0; index2 < mdimension; index2++)
        {
          if (playerturn%2)
            {
              if (index2 == element)
                {
                  if (buttonell[index1][index2]->text().toInt() > 0)
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::green));
                    }
                  else
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::red));
                    }
                }
              else
                {
                  if (buttonell[index1][index2]->text().toInt() > 0)
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::darkGreen));
                    }
                  else
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::darkRed));
                    }
                }
            }
	  else
            {
              if (index1 == element)
                {
                  if (buttonell[index1][index2]->text().toInt() > 0)
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::green));
                    }
                  else
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::red));
                    }
                }
              else
                {
                  if (buttonell[index1][index2]->text().toInt() > 0)
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::darkGreen));
                    }
                  else
                    {
                      buttonell[index1][index2]->setPalette(QPalette(Qt::darkRed));
                    }
                }
            }
          if (m_board->getElement(index1,index2) == 0)
            {
              buttonell[index1][index2]->setText("");
              buttonell[index1][index2]->setPalette(QPalette(Qt::gray));
            }


        }
    }
}
